declare function defaultNoopBatch(callback: () => void): void;
export declare const setBatch: (newBatch: typeof defaultNoopBatch) => typeof defaultNoopBatch;
export declare const getBatch: () => typeof defaultNoopBatch;
export {};
