import { CreateApi } from '../createApi';
import { coreModule, coreModuleName } from './module';
declare const createApi: CreateApi<typeof coreModuleName>;
export { createApi, coreModule, coreModuleName };
