"use strict";

exports.__esModule = true;
Object.defineProperty(exports, "unstable_batchedUpdates", {
  enumerable: true,
  get: function () {
    return _reactNative.unstable_batchedUpdates;
  }
});

var _reactNative = require("react-native");