import type { useSyncExternalStore } from 'use-sync-external-store';
import type { useSyncExternalStoreWithSelector } from 'use-sync-external-store/with-selector';
export declare const notInitialized: () => never;
export declare type uSES = typeof useSyncExternalStore;
export declare type uSESWS = typeof useSyncExternalStoreWithSelector;
