import type { Action, Dispatch } from 'redux';
export declare function createInvalidArgFactory(arg: unknown, name: string): (dispatch: Dispatch<Action<unknown>>, options: {
    readonly wrappedComponentName: string;
}) => never;
