/// <reference types="hoist-non-react-statics" />
import * as React from 'react';
import { FormikContextType } from './types';
export interface ErrorMessageProps {
    id?: string;
    name: string;
    className?: string;
    component?: string | React.ComponentType;
    children?: (errorMessage: string) => React.ReactNode;
    render?: (errorMessage: string) => React.ReactNode;
}
export declare const ErrorMessage: React.FC<ErrorMessageProps> & import("hoist-non-react-statics").NonReactStatics<React.ComponentClass<ErrorMessageProps & {
    formik: FormikContextType<ErrorMessageProps & {
        formik: FormikContextType<any>;
    }>;
}, any>, {}>;
