"use strict";(self["alertingDashboards_bundle_jsonpfunction"]=self["alertingDashboards_bundle_jsonpfunction"]||[]).push([["21"],{87354:function(__unused_webpack_module,__webpack_exports__,__webpack_require__){__webpack_require__.d(__webpack_exports__,{A:()=>esm_useObservable});var external_osdSharedDeps_React_=__webpack_require__(46933);var noop=function(){};function on(obj){var args=[];for(var _i=1;_i<arguments.length;_i++){args[_i-1]=arguments[_i]}if(obj&&obj.addEventListener){obj.addEventListener.apply(obj,args)}}function off(obj){var args=[];for(var _i=1;_i<arguments.length;_i++){args[_i-1]=arguments[_i]}if(obj&&obj.removeEventListener){obj.removeEventListener.apply(obj,args)}}var isBrowser=typeof window!=="undefined";var isNavigator=null&&typeof navigator!=="undefined";var useIsomorphicLayoutEffect=isBrowser?external_osdSharedDeps_React_.useLayoutEffect:external_osdSharedDeps_React_.useEffect;const esm_useIsomorphicLayoutEffect=useIsomorphicLayoutEffect;function useObservable(observable$,initialValue){var _a=(0,external_osdSharedDeps_React_.useState)(initialValue),value=_a[0],update=_a[1];esm_useIsomorphicLayoutEffect(function(){var s=observable$.subscribe(update);return function(){return s.unsubscribe()}},[observable$]);return value}const esm_useObservable=useObservable},3412:function(__unused_webpack_module,__webpack_exports__,__webpack_require__){__webpack_require__.r(__webpack_exports__);__webpack_require__.d(__webpack_exports__,{"default":()=>SavedObjectsTablePage});var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__=__webpack_require__(31085);var react__WEBPACK_IMPORTED_MODULE_1__=__webpack_require__(46933);var react__WEBPACK_IMPORTED_MODULE_1___default=__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);var lodash__WEBPACK_IMPORTED_MODULE_2__=__webpack_require__(81731);var lodash__WEBPACK_IMPORTED_MODULE_2___default=__webpack_require__.n(lodash__WEBPACK_IMPORTED_MODULE_2__);var _osd_i18n__WEBPACK_IMPORTED_MODULE_3__=__webpack_require__(47528);var _osd_i18n__WEBPACK_IMPORTED_MODULE_3___default=__webpack_require__.n(_osd_i18n__WEBPACK_IMPORTED_MODULE_3__);var react_use__WEBPACK_IMPORTED_MODULE_6__=__webpack_require__(87354);var _objects_table__WEBPACK_IMPORTED_MODULE_4__=__webpack_require__(44949);var _utils__WEBPACK_IMPORTED_MODULE_5__=__webpack_require__(72010);var SavedObjectsTablePage=function(param){var coreStart=param.coreStart,dataStart=param.dataStart,allowedTypes=param.allowedTypes,serviceRegistry=param.serviceRegistry,actionRegistry=param.actionRegistry,columnRegistry=param.columnRegistry,namespaceRegistry=param.namespaceRegistry,setBreadcrumbs=param.setBreadcrumbs,dataSourceEnabled=param.dataSourceEnabled,dataSourceManagement=param.dataSourceManagement,navigation=param.navigation,useUpdatedUX=param.useUpdatedUX,isDatasetManagementEnabled=param.isDatasetManagementEnabled;var capabilities=coreStart.application.capabilities;var itemsPerPage=coreStart.uiSettings.get("savedObjects:perPage",50);var dateFormat=coreStart.uiSettings.get("dateFormat");var currentWorkspace=(0,react_use__WEBPACK_IMPORTED_MODULE_6__.A)(coreStart.workspaces.currentWorkspace$);(0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(function(){setBreadcrumbs([useUpdatedUX?currentWorkspace?{text:_osd_i18n__WEBPACK_IMPORTED_MODULE_3__.i18n.translate("savedObjectsManagement.updatedUX.workspace.title",{defaultMessage:"Workspace assets"})}:{text:_osd_i18n__WEBPACK_IMPORTED_MODULE_3__.i18n.translate("savedObjectsManagement.updatedUX.title",{defaultMessage:"Assets"})}:{text:_osd_i18n__WEBPACK_IMPORTED_MODULE_3__.i18n.translate("savedObjectsManagement.breadcrumb.index",{defaultMessage:"Saved objects"}),href:"/"}])},[setBreadcrumbs,useUpdatedUX,currentWorkspace]);return(0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_objects_table__WEBPACK_IMPORTED_MODULE_4__.W,{allowedTypes:allowedTypes,serviceRegistry:serviceRegistry,actionRegistry:actionRegistry,columnRegistry:columnRegistry,namespaceRegistry:namespaceRegistry,savedObjectsClient:coreStart.savedObjects.client,indexPatterns:dataStart.indexPatterns,search:dataStart.search,http:coreStart.http,overlays:coreStart.overlays,notifications:coreStart.notifications,applications:coreStart.application,workspaces:coreStart.workspaces,perPageConfig:itemsPerPage,goInspectObject:function(savedObject){var inAppUrl=(0,_utils__WEBPACK_IMPORTED_MODULE_5__.v)(savedObject,coreStart);if(inAppUrl){return coreStart.application.navigateToUrl(inAppUrl)}},dateFormat:dateFormat,canGoInApp:function(savedObject){var inAppUrl=savedObject.meta.inAppUrl;return inAppUrl?Boolean((0,lodash__WEBPACK_IMPORTED_MODULE_2__.get)(capabilities,inAppUrl.uiCapabilitiesPath)):false},dataSourceEnabled:dataSourceEnabled,dataSourceManagement:dataSourceManagement,navigationUI:navigation.ui,useUpdatedUX:useUpdatedUX,isDatasetManagementEnabled:isDatasetManagementEnabled})}}}]);