import type { SerializeQueryArgs } from '@reduxjs/toolkit/query';
import type { EndpointDefinition } from '@reduxjs/toolkit/query';
export declare function useStableQueryArgs<T>(queryArgs: T, serialize: SerializeQueryArgs<any>, endpointDefinition: EndpointDefinition<any, any, any, any>, endpointName: string): T;
